/**************************************************************************
* Copyright (c) 2006, 2007 Dusan Stastny. All rights reserved.            *
*                                                                         *
* This software is free software; you can redistribute it and/or modify   *
* it under the terms of the GNU General Public License as published by	  *
* the Free Software Foundation; either version 2 of the License, or       *
* (at your option) any later version.                                     *
*                                                                         *
* This software is distributed in the hope that it will be useful, but    *
* WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
* General Public License for more details.                                *
*                                                                         *
* You should have received a copy of the GNU General Public License       *
* along with DS Java Disassembler; see the file COPYING. If not, write    *
* to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,      *
* Boston, MA 02111-1307 USA.                                              *
*                                                                         *
*   Dusan Stastny                                                         *
*   Jizni Svahy 804              info@developservice.cz                   *
*   330 27 Vejprnice             http://www.developservice.cz             *
*   Czech Republic - EUROPE                                               *
**************************************************************************/

/** @file
   This file contains definitions of common constants and data structures.
   @author Dusan Stastny
*/

#include <windows.h>

#ifdef __cplusplus
extern "C" {
#endif


/* access flags */
#define ACC_PUBLIC       0x0001	   /* may be accessed from outside its package */
#define ACC_PRIVATE      0x0002	   /* usable only within the defining class */
#define ACC_PROTECTED    0x0004	   /* may be accessed within subclasses */
#define ACC_STATIC       0x0008	   /* declared static */
#define ACC_FINAL        0x0010	   /* no subclasses allowed (class)
								      no further assignment after initialization (field) */
#define ACC_SUPER        0x0020	   /* treat superclass methods specially when invoked by the invokespecial instruction */
#define ACC_SYNCHRONIZED 0x0020	   /* invocation is wrapped in a monitor lock (method) */
#define ACC_VOLATILE     0x0040	   /* cannot be cached */
#define ACC_TRANSIENT    0x0080	   /* not written or read by a persistent object manager */
#define ACC_NATIVE       0x0100	   /* implemented in a language other than Java */
#define ACC_INTERFACE    0x0200	   /* is an interface, not a class */
#define ACC_ABSTRACT     0x0400	   /* may not be instantiated (class)
								      no implementation is provided (method) */
#define ACC_STRICT       0x0800	   /* floating-point mode is FP-strict */
/* VM specific */
#define ACC_FREEMEM     0x10000    /* free memory */


/* constant pool tag meaning */
#define CONSTANT_Utf8                 1
#define CONSTANT_Integer              3
#define CONSTANT_Float                4
#define CONSTANT_Long                 5
#define CONSTANT_Double               6
#define CONSTANT_Class                7
#define CONSTANT_String               8
#define CONSTANT_FieldRef             9
#define CONSTANT_MethodRef           10
#define CONSTANT_InterfaceMethodRef  11
#define CONSTANT_NameAndType         12


/* .class file data reading */
#define getClassInt(b) (jint)((0xFF&(b)[0])<<24 | (0xFF&(b)[1])<<16 | (0xFF&(b)[2])<<8 | (0xFF&(b)[3]))
#define getClassUShort(b) (jushort)(((0xFF&(b)[0])<<8) | (0xFF&(b)[1]))

	
/* system specific primitive types */
typedef unsigned char jboolean;
typedef float  jfloat;
typedef double jdouble;
typedef char   jbyte;
#ifdef __GNUC__
	typedef int jshort __attribute__((__mode__(__HI__)));
	typedef int jint   __attribute__((__mode__(__SI__)));
	typedef int jlong  __attribute__((__mode__(__DI__)));
	typedef unsigned short jushort __attribute__((__mode__(__HI__)));
#else
	typedef __int16 jshort;
	typedef __int32 jint;
	typedef __int64 jlong;
	typedef unsigned __int16 jushort;
#endif
typedef jushort jchar;
typedef jint jsize;


/* fieldID and methodID types */
typedef struct _jfieldID_struct _jfieldID;
typedef struct _methodID_struct _jmethodID;
typedef _jfieldID *jfieldID;
typedef _jmethodID *jmethodID;


/* low memory management */
#define vmMemAlloc          malloc
#define vmMemFree(p)        free((void*)p)
#define vmMemAllocObject    vmMemAlloc
#define vmMemFreeObject     vmMemFree
#define vmMemCmp            memcmp
#define vmMemCpy            MoveMemory
#define vmMemSet(dest,ch,len) FillMemory(dest,len,ch)
#define vmMemSetZero        ZeroMemory


/* Unicode string functions */
#define vmStrCat            lstrcatW
#define vmStrCmp            lstrcmpW
#define vmStrCmpi           lstrcmpiW
#define vmStrCpy            lstrcpyW
#define vmStrnCpy           lstrcpynW
#define vmStrLen            lstrlenW
#define vmPrintf            wprintf
#define vmSprintf           wsprintfW
#define vmStrLwr            CharLowerW
#define vmStrUpr            CharUpperW


/* UTF-8 string functions */
#define vmStrCatUTF         lstrcatA
#define vmStrCmpUTF         lstrcmpA
#define vmStrCmpiUTF        lstrcmpiA
#define vmStrCpyUTF         lstrcpyA
#define vmStrnCpyUTF        lstrcpynA
#define vmStrLenUTF         lstrlenA
#define vmSprintfUTF        wsprintfA
int fromUtf8(jchar* str, const char *utf8);
int toUtf8(char *utf8, const jchar *str, int len);


/* constant pool definition structure */
typedef struct _clsConstPool {
	jbyte tag;
	void *p, *q;
	jint i, j;
} clsConstPool;


/* attribute definition structure */
typedef struct _clsAttribute {
	char *name;                    /* name UTF-8 string */
	jint len;                      /* data length */
	jbyte *data;
} clsAttribute;


/* field definition structure */
struct _clsDefinition;
typedef struct _jfieldID_struct {
	struct _clsDefinition *clazz;
	jshort access_flags;
	char *name;                    /* name UTF-8 string */
	char *descriptor;              /* descriptor UTF-8 string */
	jshort attributes_count;
	clsAttribute *attributes;
	jint class_data_off;           /* data offset of the instance fields */
} clsField;


/* method definition structure */
typedef struct _methodID_struct {
	struct _clsDefinition *clazz;
	jshort access_flags;
	char *name;                    /* name UTF-8 string */
	char *descriptor;              /* descriptor UTF-8 string */
	jshort parameters_count;
	jshort stack_size;             /* stack size (number of the VM_STACK_SLOT_TYPE) */
	char return_type;              /* return value type */
	jshort attributes_count;
	clsAttribute *attributes;
	void *code;
} clsMethod;


/* class definition structure */
typedef struct _clsDefinition {
	jushort minor_version;
	jushort major_version;
	char *name;                    /* name UTF-8 string */
	struct _clsDefinition *super_class;
	jushort constant_pool_count;
	clsConstPool *constant_pool;
	jint access_flags;
	jushort interfaces_count;
	struct _clsDefinition **interfaces;
	jushort class_fields_count;
	clsField *class_fields;        /* static fields */
	jushort instance_fields_count;
	clsField *instance_fields;     /* instance fields */
	jushort class_methods_count;
	clsMethod **class_methods;     /* static methods */
	jushort instance_methods_count;
	clsMethod **instance_methods;  /* instance methods */
	jushort attributes_count;
	clsAttribute *attributes;
	jint fields_data_size;         /* data size of this class only */
	jint total_data_size;          /* total data size (with super classes) */
} clsDefinition;


#ifdef __cplusplus
}
#endif
